<?php
/*
  Plugin Name: Google Maps Travel Route
  Description: With the Google Maps Travel Route plugin you can easily show a static route on Google Maps.
  Version: 1.0
  Author: traveler11
  Author URI: http://dev.egl.is/
  Plugin URI: http://dev.egl.is/
 */
 
global $gmr_is_script_included;
$gmr_is_script_included = FALSE;
require_once ('php/gmtr-admin.php');
require_once ('php/gmtr-widget.php');

define('GMTR_PLUGIN_URL', WP_PLUGIN_URL . '/' . dirname(plugin_basename(__FILE__)));
define('GMTR_PLUGIN_DIR', WP_PLUGIN_DIR . '/' . dirname(plugin_basename(__FILE__)));

add_action('admin_menu', 'gmtr_administration_menu');
/*
 * Administration Menu
 */

function gmtr_administration_menu() {
    add_object_page('Google Maps Travel Route', 'Travel Route', 'manage_options', 'gmtr-gmaps-page', '', GMTR_PLUGIN_URL . '/images/google-marker-gray.png');
    add_submenu_page('gmtr-gmaps-page', 'Add New Location', 'Add New Location', 'manage_options', 'gmtr-gmaps-page', 'gmtr_gmaps_page');
    add_submenu_page('gmtr-gmaps-page', 'List Locations', 'List Locations', 'manage_options', 'gmtr-list-gmaps-page', 'gmtr_list_gmaps_page');
    add_submenu_page('gmtr-gmaps-page', 'Settings', 'Settings', 'manage_options', 'gmtr-settings-gmaps-page', 'gmtr_settings_gmaps_page');
}

register_activation_hook(__FILE__, 'gmtr_gmaps_install');

/**
 * @global <type> $wpdb
 * @global string $table_gmaps
 */
function gmtr_gmaps_install() {
    global $wpdb, $table_gmaps;
    $table_gmaps = $wpdb->base_prefix . 'gmtr_data';
    if (!empty($wpdb->charset))
        $charset_collate = "DEFAULT CHARACTER SET $wpdb->charset";
    if (!empty($wpdb->collate))
        $charset_collate .= " COLLATE $wpdb->collate";

    $sql = "CREATE TABLE {$table_gmaps} (
                            id bigint(20) NOT NULL AUTO_INCREMENT PRIMARY KEY,
                            gid bigint(20),
                            orderid bigint(20),
                            `gmaps_address` longtext,
                            `gmaps_title` varchar(200),
                            `gmaps_description` longtext,
                            `gmaps_url` varchar(200) ,
                            `gmaps_lat_log` varchar(50),
                            `created_date` TIMESTAMP DEFAULT CURRENT_TIMESTAMP
                            ) {$charset_collate};";
    require_once( ABSPATH . 'wp-admin/includes/upgrade.php');
    dbDelta($sql);
}

if ($_REQUEST['page'] === 'gmtr-list-gmaps-page' && is_admin()) {
    wp_enqueue_script('gmtr-gmaps-admin', GMTR_PLUGIN_URL . '/js/gmtr-admin.js', array('jquery'));
}

add_shortcode('travelroute', 'google_map_travel_route_shortcode');

/**
 * Google Maps Travel Route Shortcode function
 */
function google_map_travel_route_shortcode($atts, $content=NULL) {
    if (get_option('gmtr_gmaps_apikey')) {
        global $wpdb, $table_gmaps, $gmr_is_script_included, $id;
        $table_gmaps = $wpdb->base_prefix . 'gmtr_data';
        $sql = "SELECT * from {$table_gmaps}";
        $data = $wpdb->get_results($sql, ARRAY_A);

        $gmr_id = 'gmr_id_' . $id;
        extract(shortcode_atts( array(
                'height' => '500',
                'width' => '640',
                'title' => ''
                ), $atts));

	// String for output
	$op = '';
        $op.='<div id="' . $gmr_id . '" style="width:' . $width . 'px;height:' . $height . 'px"></div>';	

	$op.='<script type="text/javascript">';
	$op.='google.maps.visualRefresh = true;';
	
        $op.='var map;';
	$op.='var iconBase = \''. GMTR_PLUGIN_URL .'/images/icons/\';';
        $op.='var bounds = new google.maps.LatLngBounds();';
	$op.='var routePath;';
	$op.='var infowindow = new google.maps.InfoWindow({});';
    
        $op.='function initialize() {';

	    /**************************************
	    *  General map options
	    **************************************/
            $op.='var mapOptions = {';
	        $op.='streetViewControl: false,';
	        $op.='draggable: true,';
	        $op.='mapTypeId: google.maps.MapTypeId.ROADMAP,';
	        $op.='zoom: 0,';
	        $op.='zoomControl: true,';
	        $op.='zoomControlOptions: {';
		  $op.='style: google.maps.ZoomControlStyle.SMALL,';
		  $op.='position: google.maps.ControlPosition.LEFT_TOP';
	          $op.='},';
	        //$op.='maxZoom: 17,';
	        //$op.='minZoom: 2,';
	        $op.='mapTypeControl: true,';
	        $op.='panControl: false,';
	        $op.='center: new google.maps.LatLng("47.362645", "8.541784")';
	    $op.='};';

            // Create Google Map Object 
            $op.='map = new google.maps.Map(document.getElementById("'. $gmr_id .'"), mapOptions);';

	    /**************************************
	    *  Line symbol - used for animation
	    **************************************/
	    $op.='var lineSymbol = {';
  	        //$op.='path: \'M 0,-1 0,1\',';
  	        $op.='path: google.maps.SymbolPath.CIRCLE,';
    	        $op.='scale: 2,';
    	        $op.='strokeColor: \'#fff\',';
    	        $op.='strokeOpacity: 1';
  	    $op.='};';
	    
	    /**************************************
	    *  Polyline options
	    **************************************/
            $op.='routePath = new google.maps.Polyline({';
                $op.='strokeColor: "'. get_option('gmtr_marker_color') .'",';
                $op.='strokeOpacity: '. get_option('gmtr_marker_transparent') .',';
                $op.='strokeWeight: '. get_option('gmtr_marker_width') .',';
                $op.='clickable: false,';
		// Coming soon...
		//$op.='icons: [{ icon: lineSymbol, offset: \'0\', repeat: \'20px\'}],';
		//$op.='icons: [{ icon: lineSymbol, offset: \'100%\'}],';
		$op.='map: map,';
                $op.='geodesic: true';
            $op.='});';
           
	    /**************************************
	    *  Create waypoint array
	    **************************************/
	    $op.='var pathCoordinates = new Array();';
            for($i=0;$i<count($data);$i++) {
	        $value = explode(',', $data[$i]['gmaps_lat_log']);
		$op.='pathCoordinates.push(new google.maps.LatLng("'.$value[0].'", "'.$value[1].'"));';
	    }

	    // Add pathCoordinates to map
            $op.='routePath.setPath(pathCoordinates);';

	    // Add pathCoordinates to google.maps.LatLngBounds
	    $op.='for (var i = 0; i < pathCoordinates.length; i++) {';
	    $op.='bounds.extend(pathCoordinates[i]);';
	    $op.='}';

	    /**************************************
	    *  Create markers
	    **************************************/
	    for($i=0;$i<count($data);$i++) {
		$value = explode(',', $data[$i]['gmaps_lat_log']);
	    	$op.='var marker'. $i .' = new google.maps.Marker({';
		    $op.='position: new google.maps.LatLng("'.$value[0].'", "'.$value[1].'"),';
		    $op.='title: \''. $data[$i]['gmaps_title'] .'\',';
		    $op.='icon: iconBase + \'flag-white.png\',';
		    //$op.='animation: google.maps.Animation.DROP,';
		    $op.='map: map';
		$op.='});';

	        // Add event listener for popup infobox
	        $op.='google.maps.event.addListener(marker'.$i.', \'click\', function() {';
		    $op.='var content = \'<b>'. $data[$i]['gmaps_title'] .'</b>\'';
				if($data[$i]['gmaps_description'] != '') {
				    $op.='+\'<br>'. $data[$i]['gmaps_description'] .'\'';
				}
				if($data[$i]['gmaps_url'] != '') { 
				    $op.='+\'<br><a href="'. $data[$i]['gmaps_url'] .'" target="_blank">'. $data[$i]['gmaps_url'] .'</a>\'';
				}
		    $op.=';';

		    $op.='infowindow.setContent(content);';
		    $op.='infowindow.open(map, marker'. $i .' );});';
	    }

	    // Auto zoom, auto center
	    if(!empty($data)) {
	        $op.='map.fitBounds(bounds);';
            	$op.='map.panToBounds(bounds);';
		
		// Because panToBounds is a bit too close, we will zoom out...
		$op.='zoomChangeBoundsListener = google.maps.event.addListenerOnce(map, \'bounds_changed\', function(event) {';
        	$op.='if (this.getZoom()){ this.setZoom(map.getZoom()-1);}});';
		$op.='setTimeout(function(){google.maps.event.removeListener(zoomChangeBoundsListener)}, 2000);';
	    }
	   
	    // Run animation 
	    $op.='animateLine();';
        $op.='}';

	$op.='function animateLine() {';
		$op.='var count = 0;';
		$op.='timer = window.setInterval(function() {';
		$op.='count = (count + 1) % 200;';
		$op.='var icons = routePath.get(\'icons\');';
		$op.='icons[0].offset = (count / 2) + \'%\';';
		$op.='routePath.set(\'icons\', icons);';
		// Coming soon...
		// Examples1: http://stackoverflow.com/questions/16800865/animate-google-maps-polyline
		// Examples2: http://planefinder.net/route/SFO/
		$op.='}, 100);';
		$op.='';
	$op.='}';

	$op.='google.maps.event.addDomListener(window, \'load\', initialize);';
        $op.='</script>';

        //if (!$gmr_is_script_included) {
        if (true) {
	    $content .= '<script type="text/javascript" src="https://maps.googleapis.com/maps/api/js?key=' . get_option('gmtr_gmaps_apikey') . '&sensor=false"></script>';
            $gmr_is_script_included = true;
        }
        $content.= $op;
    }
    return $content;
}

?>
